const fg = require('fast-glob').sync;

module.exports = ({
  match = ['!index.js', '**/*.js'],
  cwd = process.cwd(),
  sequelize,
}) => {
  const files = fg(match, {
    dot: false,
    cwd: cwd,
    absolute: true,
  });

  const metadata = files
    .map((file) => {
      const model = sequelize.import(file);
      model.sequelize = sequelize;
      model.Sequelize = sequelize.Sequelize;

      return {
        file,
        model,
        name: model.name,
        tableName: model.tableName,
        tableSchema: model.tableSchema,
        context: require(file.replace(/\.[^/.]+$/, "")),
      }
    });

  const models = metadata
    .reduce((acc, entry) => {
      acc[entry.model.name] = entry.model;
      return acc;
    }, {});

  metadata.forEach((entry) => {
    if (entry.model.associate) {
      entry.model.associate(models, sequelize, sequelize.Sequelize.DataTypes);
    }
  });

  return {
    metadata,
    models,
    files,
  };
};